/* ------------------------------------------------------------------
 * Un exemple d'implmentation du cache d'un fichier afin d'explorer
 * l'effet des algorithmes de gestion et de remplacement
 * ------------------------------------------------------------------
 * Jean-Paul Rigault
 * ------------------------------------------------------------------
 * Stratgie RAND (c'est--dire au hasard) donne  titre d'exemple)
 * ------------------------------------------------------------------
 */

#include <assert.h>

#include "strategy.h"
#include "low_cache.h"
#include "random.h"

/* ------------------------------------------------------------------
 * Fonctions externes (utlises par cache.c)
 * ------------------------------------------------------------------
 */

/* Initialisation de la stratgie 
 * ------------------------------ 
 *Rien  faire.
*/
void *Strategy_Create(struct Cache *pcache) 
{
    return NULL;
}

/* Fermeture de la stratgie
 * -------------------------
 * Rien  faire.
 */
void Strategy_Close(struct Cache *pcache)
{
}

/* Invalidation du cache
 * ---------------------
 * Rien  faire.
 */
void Strategy_Invalidate(struct Cache *pcache)
{
}

/* Algorithme de remplacement de bloc
 * ---------------------------------- 

 * On prend le premier bloc invalide. S'il n'y en a plus on prend un bloc au hasard.
 */
struct Cache_Block_Header *Strategy_Replace_Block(struct Cache *pcache) 
{
    int ib;
    struct Cache_Block_Header *pbh;

    /* On cherche d'abord un bloc invalide */
    if ((pbh = Get_Free_Block(pcache)) != NULL) return pbh;

    /* Sinon on tire un numro de bloc au hasard */
    ib = RANDOM(0, pcache->nblocks);
    return &pcache->headers[ib];
}

/* Fonctions "rflexes" en cas de lecture et d'criture
 * ---------------------------------------------------- 
 * Ici on ne fait rien.
 */
void Strategy_Read(struct Cache *pcache, struct Cache_Block_Header *pbh) 
{
}  
  
void Strategy_Write(struct Cache *pcache, struct Cache_Block_Header *pbh)
{
} 

/* Identification de la stratgie
 * ------------------------------
 */
char *Strategy_Name()
{
    return "RAND";
}
