/* ------------------------------------------------------------------
 * Un exemple d'implmentation du cache d'un fichier afin d'explorer
 * l'effet des algorithmes de gestion et de remplacement
 * ------------------------------------------------------------------
 * Jean-Paul Rigault --- ESSI --- Janvier 2005
 * ------------------------------------------------------------------
 * Interface utilisateur
 * ------------------------------------------------------------------
 */

#ifndef _CACHE_H_
#define _CACHE_H_

#include <stdlib.h>

/* Note : ce fichier (et donc l'utilisateur du cache, n'a pas besoin de
 * connaitre les dtails de l'implmentation du cache (struct Cache) : c'est un
 * exemple de type *opaque* 
 */

/* Cration du cache */
struct Cache *Cache_Create(const char *fic,	/* le nom du ficheir */
                    unsigned nblocks,	/* nombre de blocs dans le cache */
                    unsigned nrecords,	/* nombre d'enregistrements par bloc */
                    size_t recordsz,	/* taille d'un enregistrement */
                    unsigned nderef);	/* priode de drfrenage pour NUR */

/* Fermeture (destruction) du cache */
int Cache_Close(struct Cache *pcache);

/* Synchronisation du cache */
int Cache_Sync(struct Cache *pcache);

/* Invalidation du cache */
int Cache_Invalidate(struct Cache *pcache);

/* Lecture et criture ( travers le cache) */
int Cache_Read(struct Cache *pcache, int irfile, void *precord);
int Cache_Write(struct Cache *pcache, int irfile, const void *precord);

/* Instrumentation du cache */
struct Cache_Instrument
{
    unsigned n_reads; 	/* Nombre de lectures */
    unsigned n_writes;	/* Nombre d'critures */
    unsigned n_hits;	/* Nombre de fois o l'lment tait dj dans le cache */
    unsigned n_syncs;	/* Nombre d'appels  Cache_Sync() */
    unsigned n_deref;	/* Nombre de drfrenage (NUR) */
};

struct Cache_Instrument *Cache_Get_Instrument(struct Cache *pcache);

/* Erreurs */
#define CACHE_OK 1
#define CACHE_KO 0

#endif /* _CACHE_H_ */
